import { context } from './context';
export const defaultSuite = suite('');
export const test = (name, fn) => (context.currentSuite || defaultSuite).test(name, fn);
export function clearContext() {
    context.suites.length = 0;
    defaultSuite.clear();
    context.currentSuite = defaultSuite;
}
function processSuite(mode, suiteName, factory) {
    const queue = [];
    const factoryQueue = [];
    const suite = {
        name: suiteName,
        mode,
        test,
        collect,
        clear,
    };
    function test(name, fn) {
        const task = {
            suite,
            name,
            fn,
        };
        queue.push(task);
    }
    function clear() {
        queue.length = 0;
        factoryQueue.length = 0;
    }
    async function collect() {
        factoryQueue.length = 0;
        if (factory)
            await factory(test);
        return [...factoryQueue, ...queue];
    }
    context.currentSuite = suite;
    context.suites.push(suite);
    return suite;
}
export function suite(suiteName, factory) {
    return processSuite('run', suiteName, factory);
}
suite.skip = function skip(suiteName, factory) {
    return processSuite('skip', suiteName, factory);
};
suite.only = function skip(suiteName, factory) {
    return processSuite('only', suiteName, factory);
};
suite.todo = function skip(suiteName) {
    return processSuite('todo', suiteName);
};
// alias
export const describe = suite;
export const it = test;
